<?php

namespace Roberto\Roberto\Providers;

use Illuminate\Support\ServiceProvider;

class RobertoServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'roberto');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'roberto');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerRoberto();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerRoberto()
    {
        $this->app->bind('roberto', function($app) {
            return new Roberto($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Roberto', 'Lavalite\Roberto\Facades\Roberto');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('roberto.roberto', function ($app) {
            return $this->app->make('Roberto\Roberto\Roberto');
        });

        // Bind 1 to repository
        $this->app->bind(
            'Roberto\Roberto\Interfaces\1RepositoryInterface',
            \Roberto\Roberto\Repositories\Eloquent\1Repository::class
        );        // Bind 1 to repository
        $this->app->bind(
            'Roberto\Roberto\Interfaces\1RepositoryInterface',
            \Roberto\Roberto\Repositories\Eloquent\1Repository::class
        );        // Bind 2 to repository
        $this->app->bind(
            'Roberto\Roberto\Interfaces\2RepositoryInterface',
            \Roberto\Roberto\Repositories\Eloquent\2Repository::class
        );

        $this->app->register(\Roberto\Roberto\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Roberto\Roberto\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and roberto's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/roberto.php', 'roberto'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeRoberto::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['roberto.roberto'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('roberto/roberto.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/roberto')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/roberto')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
